function doTrain(baseName, mode)
%% main training function
% author: dino bellugi
% copyright: blueG soft wear 2010
% takes in the data structure with labels and regions list
% expects that descriptors have been already generated for the entire tree
% calls the SVM using 'search for best model' option
% trains with 'generate probabilities option'
% saves resulting model
% extracts the approriate regions and descriptors before calling trainSVM
%
% mode is an optional logical indicating if the descriptors should be
% generated (true) or is they alread exist (false, default).
%
% contains some hard-coded information:
%	params = parameters for training
%       params(1): 0=load model, 1=find best parameters, 2=use defaults
%       params(2): 0=linear, 1=poly, 2=rbf, 3=sigmoid
%       params(3): minimum degree (for poly only)
%       params(4): maximum degree (for poly only)
%       params(5): number of subsets for cross-validation
%       params(6): lower bound of C search space (log)
%       params(7): upper bound of C search space (log)
%       params(8): lower bound of G search space (log)
%       params(9): upper bound of G search space (log)
%       params(10): coarse search step (in the log space)
%       params(11): fine search step (in the log space)
%   options = logical(1,3) for print, save, test

%%
    % check command line
    if (nargin < 1 || nargin > 2)
        error('Usage: doTrain(baseName, [mode])');
    end
    
    if (nargin < 2)
        mode = false;
    end
    
    % add dir paths
    addLibs();
        
    % set up file names
    labelsName = [baseName '_labels.mat'];
    descriptorsName = [baseName '_descriptors'];
    trainName = [baseName '_trainData.mat'];
    
    % hard-coded parameters
    params = [1 1 1 5 5 -30 30 -30 30 1 0.25]; % poly
%     params = [1 1 2 5 5 -30 30 -30 30 1 0.25]; % rbf
    options = true(1,3);
    
    % load labels data structure
    load(labelsName);

    % make descriptors (these might be already made, so check)
    if (mode)
        fprintf(['creating descriptors for ', baseName, ' ...\n']);
        % options is 1x6 print plot save (high level) print plot save (low level)
        % save option must be on as the exchange is via file
        tic
        createDescriptors(baseName, [1 0 1 0 0 0], regList);
        toc
    else
        load(descriptorsName);
    end
   
    % unpack the descriptors
    trainDescriptors = descriptors(:, regList);
    
    % note: change zeros to -1 eventually
    %trainLabels = double(labels == 1);
    trainLabels = double(labels);
    
%     % balance the labels?
%     tempPos = (tempLabels == 1);
%     tempNeg = (tempLabels == -1);
%     
%     tempRatio = floor(numel(tempNeg) / numel(tempPos);
%     
%     ii = 1;
%     for i = 1:numel(tempPos)
%         trainLabels(ii) = tempLabels(tempPos(i));
%         trainLabels(ii+1:ii+tempRatio) = tempLabels(tempNeg((i-1)*tempRatio+1:tempNeg((i-1)*tempRatio+1)));
%         ii = ii+tempRatio;
%     end

    % save train data
    save(trainName, 'trainLabels', 'trainDescriptors');

    % call trainSVM
    [svmModel, svmType, svmParams, svmOpts, labels, accuracy, value] = ...
        trainSVM(baseName, params, options);
    
end
