function [regions, parent, scale] = extract_regions_ucm2(ucm2, th_min)
% syntax:
%   [regions, parent, scale] = extract_regions_ucm2(ucm2, th_min)
%
% description:
%   reconstruct region tree from ucm. considers all levels in ucm down to
%   superpixel scale (th_min)
%
% arguments:
%   ucm2 :      ucm 'doubleSize' format, uint8
%   th_min:     minimal scale considered, in [ 0 255]. (default 25)
%
% outputs:
%   regions: structure with binary masks of all regions
%   parent: parent node of the region in tree
%   scale: scale of appearance of the region in hierarchy
%
% Pablo Arbelaez <arbelaez@eecs.berkeley.edu>
% December 2008
    % add paths
    addLibs();
    
if nargin<2, th_min = 25; end

labels_old = ones(size(ucm2(2:2:end, 2:2:end)));
ct = 1;
regions{1} = logical(labels_old);
parent(1) = 1;
id_old(1) = 1;

for th = 255 : -1 : th_min,
    
    labels2 = bwlabel(ucm2 <= th);
    labels_new = labels2(2:2:end, 2:2:end);
    
    tot_new = max(labels_new(:));
    id_new = zeros(tot_new,1);
    gt_segs{1} = labels_old;
    
    [matches] = match_segmentations(labels_new, gt_segs);
    
    for l = 1:tot_new
       [mxv, mxl] = max(matches(l, :));
       
       if mxv == 1,
           id_new(l) = id_old(mxl);
           scale(id_old(mxl)) = th;
       elseif mxv>0,
           ct = ct + 1; 
           parent(ct) = id_old(mxl);
           regions{ct} = (labels_new==l);
           id_new(l) = ct;
           scale(ct) = th;
       end           
    end

    labels_old = labels_new;
    id_old = id_new;
end

scale = uint8(scale);
parent = uint16(parent);

%%
function [matches ] = match_segmentations(seg, gt_segs)
% modification of PASCAL evaluation code

total_gt = 0;
for s = 1: numel(gt_segs)
    total_gt = total_gt + max(gt_segs{s}(:));
end

cnt = 0;
matches = zeros(total_gt, max(seg(:)));
for s = 1 : numel(gt_segs)
    gt = gt_segs{s};

    num1 = max(gt(:)) + 1;
    num2 = max(seg(:)) + 1;
    confcounts = zeros(num1, num2);

    sumim = 1 + gt + seg*num1;

    hs = histc(sumim(:), 1:num1*num2);
    confcounts(:) = confcounts(:) + hs(:);

    accuracies = zeros(num1-1, num2-1);
    for j = 1 : num1
        for i = 1 : num2
            gtj = sum(confcounts(j, :));
            resj = sum(confcounts(:, i));
            gtjresj = confcounts(j, i);
            accuracies(j, i) = gtjresj / (gtj + resj - gtjresj);
        end
    end

    matches(cnt+1:cnt+max(gt(:)), :) = accuracies(2:end, 2:end);
    
    cnt = cnt + max(gt(:));
end
matches = matches';





