function processImage(baseName)
% author: dino bellugi
% copyright: blueG soft wear 2010
% processes image, producing gPb and UCM maps
% inputs:
%   baseName = base name for files
% example:
%   processImage('../newData/Img0057_cropped')
% notes:
%   expects gPb to be in libary path, use addpath('../lib') before calling
%   needs memory and time, do a clear all before calling
%   last image was 0158, new image is 0057
% naming convention:
%   imageName = [baseName '.bmp'];
%   ucmName = [baseName '_ucm.bmp'];
%   ucm2Name = [baseName '_ucm2.bmp'];
%   regionsName = [baseName '_regions'];
%   gPbName = [baseName '_gPb.mat'];

    %% check command line
    if (nargin ~= 1)
        error('Usage: processImage(baseName)\n');
    end
    
    %% add paths
    addLibs();
    
    %% setup naming structure
    imageName = [baseName '.bmp'];
    ucmName = [baseName '_ucm.bmp'];
    ucm2Name = [baseName '_ucm2.bmp'];
    regionsName = [baseName '_regions'];
    gPbName = [baseName '_gPb.mat'];

    %% compute globalPb
    imgFile = imageName;
    outFile = gPbName;
    rsz = 1.0;
    fprintf('Calling globalPb\n');
    tic
    globalPb(imgFile, outFile, rsz);
    toc
    fprintf('Finished globalPb\n');

    %% compute Hierarchical Regions
    load(gPbName, 'gPb_orient', 'gPb_thin');

    % boundaries
    fprintf('Calling contours2ucm\n');
    tic
    ucm = contours2ucm(gPb_orient, 'imageSize');
    toc
    fprintf('Finished contours2ucm\n');
    imwrite(ucm,ucmName);

    % regions
    fprintf('Calling contours2ucm\n');
    tic
    ucm2 = contours2ucm(gPb_orient, 'doubleSize');
    toc
    fprintf('Finished contours2ucm\n');
    imwrite(ucm2,ucm2Name);
    clear gPb_orient

    %% extract regions from ucm:
    ucm2=imread(ucm2Name);
    ucm = ucm2(2:2:end, 2:2:end);

    thresh=1;
    fprintf('Calling extract_regions_ucm2\n');
    tic
    [regions, parent, scale] = extract_regions_ucm2(ucm2,thresh);
    toc
    fprintf('Finished extract_regions_ucm2\n');
    
    save(regionsName, 'regions', 'parent', 'scale', 'thresh', 'ucm', 'ucm2', 'gPb_thin');

%% descriptors
    % make descriptors (these might be already made, so check)
    fprintf(['creating descriptors for ', baseName, ' ...\n']);
    % options is 1x6 print plot save (high level) print plot save (low level)
    % save option must be on as the exchange is via file
    tic
    createDescriptors(baseName, [1 0 1 0 0 0]);
    toc
 
end
