function S = sortPaths(S, pSort, sMode)
% sorts paths in the tree based on the weights from pSort vector:
% pSort = [perNum, perMin, perMax, perAvg], sum(pSort) = 1
% sMode = {0: ascending, 1: descending}
% notes: 
%   - all path measures are first normalized
%   - default: sort based on descending maximum distance (check if best way)
% author: db

    % check args and set defaults
    if (nargin < 1)
        error('sortPaths: insufficient parameters!\n');
    end
    if (nargin < 2 || numel(pSort ~= 4) || sum(pSort) ~= 1)
        pSort = [0 0 1 0];
    end
    if (nargin < 3 || sMode < 0 || sMode > 1)
        pMode = true;
    end
    if (pMode)
        mode = 'descend';
    else
        mode = 'ascend';
    end
    
    % start timer
    fprintf('Sorting paths ...\n');
    tStart = tic;
    
    % create sorting measure
    if (~pSort(1) || ~sum(S.paths.numNodes))
       numMes = 0;
    else
       numMes = pSort(1) .* (double(S.paths.numNodes) ./ sum(S.paths.numNodes));
    end
    if (~pSort(2) || ~sum(S.paths.minDist))
        minMes = 0;
    else
        minMes = pSort(2) .* (double(S.paths.minDist) ./ sum(S.paths.minDist));
    end
    if (~pSort(3) || ~sum(S.paths.maxDist))
        maxMes = 0;
    else
        maxMes = pSort(3) .* (double(S.paths.maxDist) ./ sum(S.paths.maxDist));
    end
    if (~pSort(4) || ~sum(S.paths.avgDist))
        avgMes = 0;
    else
        avgMes = pSort(4) .* (double(S.paths.avgDist) ./ sum(S.paths.avgDist));
    end
    S.paths.pathValue = numMes + minMes + maxMes + avgMes;
    
    % sort
    [value index] = sort(S.paths.pathValue, mode);
    S.paths.sortedPaths = uint16(index);
    
    % stop timer
    toc(tStart)
end
